=== modified file 'bzrlib/benchmarks/bench_xml.py'
--- bzrlib/benchmarks/bench_xml.py	2006-08-16 21:19:42 +0000
+++ bzrlib/benchmarks/bench_xml.py	2006-08-16 21:28:52 +0000
@@ -27,9 +27,11 @@
         # On jam's machine, ElementTree serializer took: 2161ms/13487ms
         #                      with Robert's serializer:  631ms/10770ms
         #                      with Entity escaper:       487ms/11636ms
+        #           caching Entity escaper, empty cache:  448ms/ 9489ms
         # Really all we want is a real inventory
         tree = self.make_kernel_like_committed_tree('.', link_bzr=True)
 
+        xml5._clear_cache()
         # We want a real tree with lots of file ids and sha strings, etc.
         self.time(xml5.serializer_v5.write_inventory_to_string,
                   tree.basis_tree().inventory)
@@ -38,6 +40,7 @@
         # Really all we want is a real inventory
         tree = self.make_kernel_like_committed_tree('.', link_bzr=True)
 
+        xml5._clear_cache()
         f = open('kernel-like-inventory', 'wb')
         try:
             # We want a real tree with lots of file ids and sha strings, etc.

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2006-08-16 21:19:42 +0000
+++ bzrlib/xml5.py	2006-08-16 21:28:52 +0000
@@ -27,8 +27,6 @@
 from bzrlib.errors import BzrError
 
 
-_unicode_to_escaped_map = {}
-
 _utf8_re = None
 _utf8_escape_map = {
     "&":'&amp;',
@@ -72,18 +70,24 @@
         return "&#%d;" % ord(match.group())
 
 
+_unicode_to_escaped_map = {}
+
 def _encode_and_escape(unicode_str, _map=_unicode_to_escaped_map):
     """Encode the string into utf8, and escape invalid XML characters"""
-    try:
-        return _map[unicode_str]
-    except KeyError:
+    text = _map.get(unicode_str)
+    if text is None:
         # The alternative policy is to do a regular UTF8 encoding
         # and then escape only XML meta characters. This could take
         # advantage of cache_utf8 since a lot of the revision ids
         # and file ids would already be cached.
         text = str(_utf8_re.sub(_utf8_escape_replace, unicode_str))
         _map[unicode_str] = text
-        return text
+    return text
+
+
+def _clear_cache():
+    """Clean out the unicode => escaped map"""
+    _unicode_to_escaped_map.clear()
 
 
 class Serializer_v5(Serializer):
@@ -116,6 +120,9 @@
             self._append_entry(output, ie)
         output.append('</inventory>\n')
         f.writelines(output)
+        # Just to keep the cache from growing without bounds
+        # but we may actually not want to do clear the cache
+        _clear_cache()
 
     def _append_inventory_root(self, output, inv):
         """Append the inventory root to output."""
@@ -164,9 +171,7 @@
 
     def _append_utf8_escaped(self, output, a_string):
         """Append a_string to output as utf8."""
-        #output.append(_encode_and_escape(a_string))
-        text = str(_utf8_re.sub(_utf8_escape_replace, a_string))
-        output.append(text)
+        output.append(_encode_and_escape(a_string))
         output.append('"')
 
     def _pack_inventory(self, inv):

